/*  Misfit Model 3D
 * 
 *  Copyright (c) 2004 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *  See the COPYING file for full license text.
 */


#include "startprompt.h"
#include "3dmprefs.h"
#include "viewwin.h"
#include "model.h"
#include "helpwin.h"

#include <qapplication.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qaccel.h>

#include "pixmap/mm3dlogo-32x32.xpm"

StartPrompt::StartPrompt( QWidget * parent, const char * name )
   : StartPromptBase( parent, name, WDestructiveClose, true ),
     m_accel( new QAccel(this) ),
     m_shouldExit( true )
{
   setIcon( (const char **) mm3dlogo_32x32_xpm );
   setIconText( "Misfit Model 3D" );

   m_mruList->setSorting( -1 );

   m_accel->insertItem( Key_F1, 0 );
   connect( m_accel, SIGNAL(activated(int)), this, SLOT(helpNowEvent(int)) );

   m_mruList->clear();
   QListViewItem * item = NULL;
   for ( unsigned i = 0; i < g_prefs("mru").count(); i++ )
   {
      item = new QListViewItem( m_mruList, item );
      item->setText( 0, g_prefs("mru")[i] );

      if ( i == 0 )
      {
         item->setSelected( true );
      }
   }

   updateButtons();
}

StartPrompt::~StartPrompt()
{
}

void StartPrompt::helpNowEvent( int )
{
   HelpWin * win = new HelpWin( "olh_startprompt.html", true );
   win->show();
}


void StartPrompt::openSelectedClicked()
{
   bool opened = false;
   for ( QListViewItem * item = m_mruList->firstChild(); item != NULL; item = item->nextSibling() )
   {
      if ( item->isSelected() )
      {
         if ( ViewWindow::openModel( item->text(0).latin1() ) )
         {
            opened = true;
         }
      }
   }

   if ( opened )
   {
      m_shouldExit = false;
      close();
   }
}

void StartPrompt::openFileClicked()
{
   if ( ViewWindow::openModelDialog() )
   {
      m_shouldExit = false;
      close();
   }
}

void StartPrompt::newModelClicked()
{
   ViewWindow * win = new ViewWindow( new Model );
   win->getSaved(); // Just so I don't have a warning
   m_shouldExit = false;
   close();
}

void StartPrompt::exitClicked()
{
   close();
}

void StartPrompt::updateButtons()
{
   if ( m_mruList->selectedItem() )
   {
      m_openSelectedButton->setEnabled( false );
      m_openSelectedButton->setDefault( false );
      m_newModelButton->setDefault( true );
   }
   else
   {
      m_openSelectedButton->setEnabled( true );
      m_openSelectedButton->setDefault( true );
      m_newModelButton->setDefault( false );
   }
}

