/*  Misfit Model 3D
 * 
 *  Copyright (c) 2004 Kevin Worcester
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 *
 *  See the COPYING file for full license text.
 */


#include "texture.h"

int Texture::s_allocated = 0;

#include <stdio.h>
#include <stdlib.h>

Texture::Texture()
   : m_name( NULL ),
     m_filename( NULL ),
     m_height( 0 ),
     m_width( 0 ),
     m_format( FORMAT_RGBA ),
     m_data( NULL )
{
   s_allocated++;
}

Texture::~Texture()
{
   if ( m_name )
   {
      free( m_name );
   }
   if ( m_filename )
   {
      free( m_filename );
   }
   if ( m_data )
   {
      delete[] m_data;
   }
   s_allocated--;
}

const char * Texture::errorToString( Texture::Error e )
{
   switch ( e )
   {
      case ERROR_NONE:
         return "Success";
      case ERROR_NO_FILE:
         return "File does not exist";
      case ERROR_ACCESS_DENIED:
         return "Permission denied";
      case ERROR_FILE_OPEN:
         return "Could not open file";
      case ERROR_FILE_READ:
         return "Could not read from file";
      case ERROR_BAD_MAGIC:
         return "File is the wrong type or corrupted";
      case ERROR_UNSUPPORTED_VERSION:
         return "Unsupported version";
      case ERROR_BAD_DATA:
         return "File contains invalid data";
      case ERROR_UNEXPECTED_EOF:
         return "Unexpected end of file";
      case ERROR_UNSUPPORTED_OPERATION:
         return "This operation is not supported";
      case ERROR_BAD_ARGUMENT:
         return "Invalid argument (internal error, probably null pointer argument)";
      case ERROR_UNKNOWN:
         return "Unknown error";
      default:
         break;
   }
   return "Invalid error code";
}
